module Calibration
implicit none
    contains
    

subroutine read_calibration_data
  use  constants_and_variables, ONLY: data_date,data_erosion,data_number,data_runoff,data_mgsed,data_mgh2o, &
    CalibrationDataUnit 
    use utilities
    implicit none
    
    real    :: dummy, read_runoff, read_erosion, mg_sed, mg_h2o
    integer :: day,month,year, n, i,j
    
    
    
    data_date = 0
    data_erosion = 0.0
    
  !  OPEN(Unit=88,FILE='P1data.txt',STATUS='OLD')
    read (CalibrationDataUnit,*) n
    read (CalibrationDataUnit,*)
    j=0
    do i = 1, n
       read (CalibrationDataUnit,*) month,day,year, dummy, read_runoff, read_erosion, mg_sed, mg_h2o         
       if (read_runoff > 0.0) then
           j=j+1
        data_date(j) = jd (YEAR,MONTH,DAY)
        data_erosion(j) = read_erosion
        data_runoff(j) = read_runoff
        data_mgsed(j) = mg_sed
        data_mgh2o(j) = mg_h2o
       end if
    end do
    
    data_number = j

end subroutine read_calibration_data
    
    
!*********************************************************************************    
subroutine write_calibration_data
use  constants_and_variables, ONLY: data_date,data_erosion, model_erosion,startday,data_number,data_runoff,model_runoff,  &
                         model_mg_ro, model_mg_er ,data_mgsed,data_mgh2o ,AFIELD,CalibrationFilename
use utilities
implicit none

integer :: i,YEAR,MONTH,DAY
real :: mo_mg_ro, mo_mg_er, diff_mg_ro,   diff_mg_er, r2_mg_ro,   r2_mg_er 

real :: total_data_ro, total_model_ro
real :: total_data_er, total_model_er
    
    
     do i= 1, 100

             write(97,*) i, model_mg_ro(i)

     end do




 OPEN(Unit=66,FILE= CalibrationFilename(1:len_trim(CalibrationFilename)-4)//  '_CalibrateOut.txt' , STATUS='UNKNOWN')
       
    
 write (66,*) "                     Sum Abs. Diff.   Greatest Value   Location" 
 write (66,*) "Runoff  Calibration ", sum(abs(data_runoff(1:data_number)- model_runoff(1:data_number))), &
                                         maxval(abs(data_runoff(1:data_number)- model_runoff(1:data_number))), &
                                         maxloc(abs(data_runoff(1:data_number)- model_runoff(1:data_number)))
    
 write (66,*) "Erosion Calibration ", sum(abs(data_erosion(1:data_number)- model_erosion(1:data_number))), &
                                         maxval(abs(data_erosion(1:data_number)- model_erosion(1:data_number))),&
                                         maxloc(abs(data_erosion(1:data_number)- model_erosion(1:data_number)))
    
 write(66,'(a120)') &
 'Date  |             EROSION             |          RUNOFF                |           Runoff Mass (mg)   |       EROSION Mass (mg) '
 write(66,'(a145)') &
 'Date         Data       Model       Diff     Data      Model        Diff       Data         Model       Diff         Data        Model       Diff'
    
 r2_mg_ro = 0.
 r2_mg_er = 0.
 total_data_ro = 0.
 total_data_er = 0.    
 total_model_ro = 0.
 total_model_er = 0.
 
 
 
 do i = 1, data_number
          
         mo_mg_ro = model_mg_ro(i) * 1.0e11 * AFIELD 
         mo_mg_er = model_mg_er(i) * 1.0e11 * AFIELD       
 
         
      
         if(mo_mg_ro  < 1.0e-6) then
             mo_mg_ro = 0.
         end if
           
         if(mo_mg_er  < 1.0e-6) then
          !   mo_mg_er = 0.
         end if

         total_model_ro = total_model_ro + mo_mg_ro
         total_model_er = total_model_er + mo_mg_er
        

         !-99 means that no data was gathered 
         if (data_mgh2o(i) >-99) then
             diff_mg_ro = data_mgh2o(i) - mo_mg_ro
             r2_mg_ro = r2_mg_ro + diff_mg_ro*diff_mg_ro
             total_data_ro = total_data_ro + data_mgh2o(i)
         else
             diff_mg_ro =-9999
         end if
        
         if (data_mgsed(i) >-99) then
             diff_mg_er = data_mgsed(i) - mo_mg_er
             r2_mg_er = r2_mg_er + diff_mg_er*diff_mg_er
             total_data_er = total_data_er + data_mgsed(i) 
         else
             diff_mg_er = -9999
         end if
        

        call get_date (data_date(i), YEAR,MONTH,DAY)
        write(66,'(3I3, 1X, 4(G13.7,G13.7, G11.3, 2X))') Month,day,YEAR, &
                 data_erosion(i), model_erosion(i), data_erosion(i)- model_erosion(i),&
                 data_runoff(i), model_runoff(i),  data_runoff(i)- model_runoff(i),  &
                 data_mgh2o(i), mo_mg_ro, diff_mg_ro, & 
                 data_mgsed(i), mo_mg_er, diff_mg_er
 
 end do 
 
    
    
 write (66,'(a23, G12.3)') "SSQ runoff pesticide = ", r2_mg_ro
 write (66,'(a23, G12.3)') "SSQ erosin pesticide = ", r2_mg_er
 write (66,*) 
 write (66,*) "Totals Runoff:"
 write (66,'(a23, G12.3)') "Pesticide Data (mg)  = ",  total_data_ro 
 write (66,'(a23, G12.3)') "Pesticide Model (mg) = ",  total_model_ro   
 write (66,*)                                             
 write (66,*) "Totals Erosion: "                           
 write (66,'(a23, G12.3)') "Pesticide Data (mg)  = ", total_data_er 
 write (66,'(a23, G12.3)') "Pesticide Model (mg) = ", total_model_er
    
end subroutine write_calibration_data
    
    
end module Calibration