module evapotranspiration
    implicit none
    contains

    
    SUBROUTINE EVPOTR
      use  constants_and_Variables, ONLY: NC,NCMPTS, TEMP,pfac,sttdet ,PEVP, wp,fc,SoilWater,cint,et,DELX,cevap,ncom1,ncom2,TDET 

      implicit none
!     Computes daily potential evapotranspiration,
!     canopy evaporation, and actual evapotranspiration from each
!     soil layer

      REAL         PET,PETP,ANUM,DENOM,AW,FRAC(NCMPTS),TSW
      Real         TWP,TFRAC,RNSUM,STDELX,EDEPTH
      INTEGER      ITEMP,I,NSUM


!     Compute potential evapotranspiration

      ITEMP= INT(TEMP)
      IF (ITEMP.LT.1)  ITEMP= 1
      IF (ITEMP.GT.40) ITEMP= 40
      
      !pan evaporation now forced. no Hamon option since units are wrong
      PET= PEVP*PFAC
      
!     Subtract canopy evaporation from potential evapotranspiration

      PETP= AMAX1(0.0,PET-CINT)
      IF (PET .GT. CINT) THEN
        CEVAP= CINT
      ELSE
        CEVAP= PET
      ENDIF
      CINT= CINT-CEVAP

!     Compute evapotranspiration from each soil layer
      EDEPTH = 5.0
      STDELX = DELX(1)
      STTDET = 0.0

      TDET = 0.0
      ET(1:NCOM2)= 0.0
      ANUM = 0.0
      DENOM= 0.0
      
  
      do I= 1, NCOM1
        ANUM = ANUM  + AMAX1(0.0,SoilWater(I)- WP(I))
        DENOM= DENOM + AMAX1(0.0,FC(I)- WP(I))
      end do
      
      AW   = ANUM/DENOM
      IF (AW.LT.0.6) PETP= AW*PETP/0.6
      NSUM = 0
      TSW  = 0.0
      TWP  = 0.0
   
      do I= 1, NCOM1
        NSUM = NSUM + I
        TSW  = TSW + SoilWater(I)
        TWP  = TWP + WP(I)
      end do
      
      RNSUM= FLOAT(NSUM)
      IF (RNSUM .GT. 0.00 .AND. TSW .GT. TWP) THEN
        TFRAC= 0.0
        
        do I= 1, NCOM1
          FRAC(I)=FLOAT(NCOM1-I+1)*(SoilWater(I)-WP(I))/(RNSUM*(TSW-TWP))
          TFRAC  =TFRAC + FRAC(I)
        end do
        
        
        DO  I = 1, NCOM1
          ET(I) = AMIN1((SoilWater(I)-WP(I)),PETP*FRAC(I)/TFRAC)
          ET(I) = AMAX1(ET(I),0.0)
          TDET  = TDET + ET(I)
!         This code is added by C.S.Raju to estimate the evaporation
!         through the top 5cm depth of soil.
          IF (STDELX .LE. EDEPTH)THEN
            STTDET = STTDET + ET(I)
            STDELX = STDELX + DELX(I)
          END IF

       end do

      ENDIF
    END SUBROUTINE EVPOTR
    
 end module evapotranspiration