module Hydrolgy
    implicit none
    contains
    
       
    SUBROUTINE Runoff_PRZM5
    !PRZM5: Calculate runoff first and then using any left over for the canopy & infiltration 
    !In conformance to NEH-4, initial abstraction is no longer altered by canaopy
    
      use constants_and_Variables, ONLY: NC,MXCPD,NCMPTS,EchoFileUnit,PRECIP,temp,sfac,snowfl,THRUFL,itflag,spt,IRTYPE,ncrop, &
        SoilWater,cint,DELX,CN_input,smelt,runof, cvnum,ainf,potential_canopy_holdup,theth,snow, &
        under_canopy_irrig, over_canopy_irrig, cn_moist_node, canopy_flow, &
        effective_rain ,soil_depth,  julday1900, data_date,  model_runoff    
      
      
      implicit none
!     This subroutine calculates snowmelt, crop interception, runoff, and infiltration from the soil surface

      REAL   TWLVL,CURVN,WLVL,TTLNJ,TTT,SPTTOT,SPTAVE
      INTEGER      TLIN
      
      real :: canopy_capture
      
      RUNOF = 0.0
      SMELT = 0.0
      SNOWFL= 0.0
      effective_rain = 0.0
      canopy_flow= 0.0
      
      IF (SFAC .GT. 0.0) THEN
!       Compute snowmelt and accumulation
        IF (TEMP .LE. 0.0) THEN
          SNOWFL= PRECIP
          PRECIP= 0.0
          SNOW  = SNOW+ SNOWFL
        ELSE
          SMELT = AMIN1(SFAC*TEMP,SNOW)
          SNOW  = SNOW- SMELT
        ENDIF
      ENDIF

      !**** Calculate average temperature ? **************
      SPTAVE=0.0
      SPTTOT=0.0
      IF(ITFLAG.GT.0)THEN
         TLIN  = 0
         TTLNJ = 0.0
         TTT=15.0
         do
            TLIN  = TLIN + 1
            SPTTOT=SPTTOT+SPT(TLIN)*DELX(TLIN)
            TTLNJ = TTLNJ + DELX(cn_moist_node)
            IF (TTT .LE. TTLNJ) exit      
         end do 
         SPTAVE=SPTTOT/TTLNJ
      end if
      !*****************************************
      
      !Water amount in top 10 cm used for CN manipulation
      wlvl = sum(SoilWater(1:cn_moist_node))  !amount of water in top 10 cm
      TWLVL = WLVL/ soil_depth(cn_moist_node)
      

      effective_rain  = under_canopy_irrig + over_canopy_irrig + precip + smelt  !used for runoff calc only  (canopy not included)
      
      IF (ITFLAG == 0   .OR. ((ITFLAG.GT.0).AND.(SPTAVE.GT.0.0))   ) THEN
         
         call Curve_Number_PRZM5(TWLVL,curvn)
         call Calculate_Runoff_PRZM5(curvn,Effective_Rain)
   
      ELSE IF((ITFLAG.GT.0).AND.(SPTAVE.LE.0.0))THEN
          CURVN = 100.0
          RUNOF = Effective_Rain
          write(echofileunit,*) "Frozen Ground"
      ENDIF
      
      canopy_capture = min(potential_canopy_holdup-CINT, (precip + over_canopy_irrig - runof)  )
      canopy_capture = max (0.0, canopy_capture) !for under canopy irrigation the above could be negative
       
      cint           = cint + canopy_capture
      canopy_flow = max (0.0, over_canopy_irrig + precip -canopy_capture ) !used for pesticide washoff calcs
      
      !Thrufl is now only the amount of water actually hitting the ground  used for erosion calcs
      !Might need rethinkin to remove canopy if already accounted for in C factors
      !used effective rain for time of conc in erosion routine
      THRUFL = over_canopy_irrig + precip -canopy_capture + under_canopy_irrig
      

      ! Compute infiltration for first soil compartment
      ! AINF(1) = AINF(1) + THRUFL + SMELT- RUNOF
      AINF(1) = AINF(1) + Effective_Rain- RUNOF -canopy_capture

      CVNUM = CURVN  !store curve number for later output dispakly
      
      !the following used for calibration only:
      where (data_date == julday1900)   model_runoff = runof
      
    END SUBROUTINE Runoff_PRZM5
    
    

     
     !****************************************************************************************** 
    Subroutine Curve_Number_PRZM5(twlvl,curvn)
    !moisture adjustmets to the curve number
        use constants_and_Variables, ONLY:theth,ncrop,CN_index, CN_2
        use curve_number_table
        implicit none
        real,intent(in) :: twlvl
        real,intent(out) :: curvn
        integer :: i
        

        i= CN_2(NCROP,CN_index) 
        CURVN = i + (CN_3(i) -  i) *  (TWLVL-THETH)/ THETH       
        if (twlvl < theth ) CURVN = cn_1(i)  + (i - CN_1(i))* twlvl/theth
        
        
    End Subroutine Curve_Number_PRZM5
    
     
     
     

    
    !*****************************************************************************************
    Subroutine Calculate_Runoff_PRZM5(curvn,Effective_Rain)
       use constants_and_Variables, ONLY:thrufl,RUNOF,inabs, EchoFileUnit
       implicit none
       real, intent(in) :: curvn
       real,intent(in)  :: Effective_Rain
    
       !the constant 0.508 is derived from 0.2 * 2.54 cm/in and 0.2 is from INABS = 0.2 * S, where S is (1000./CURVN-10.).
       !INABS: Initial Abstraction INABS = AMAX1(0.508* (1000./CURVN-10.),PRECIP-THRUFL)
       !THe Original NRCS Curve Number nmethod already accounts for canopy holdup.  -DFY

       INABS = 0.508* (1000./CURVN-10.)  !also used in erosion calculation
       IF (Effective_Rain .GT. INABS) then
           RUNOF= (Effective_Rain-INABS)**2/ (Effective_Rain + (4.0* INABS))  !OK the 4 is from 0.8 = 4 * 0.2
       else 
           runof = 0.0
       end if
    
    end  Subroutine Calculate_Runoff_PRZM5  
     
     
     
     
 
    
   SUBROUTINE Leaching
      use  constants_and_Variables, ONLY: NAPP,NC,NCMPTS,SoilWater,et,THETO,DELX,THETN,THEFC,ainf,ncom2,vel
      implicit none
!     Performs hydraulic calculations assuming a uniform soil profile with unrestricted drainage
!     (drainage occurs instantaneously)

      INTEGER      I

      do I=1,NCOM2
        THETO(I) = SoilWater(I)/DELX(I)
        THETN(I) = (SoilWater(I)+ AINF(I)- ET(I))/ DELX(I)
        AINF(I+1)= 0.0
        IF (THETN(I) .GT. THEFC(I)) THEN
          AINF(I+1)= (THETN(I)- THEFC(I))* DELX(I)
          THETN(I) = THEFC(I)
        ENDIF
        VEL(I)= AINF(I+1)/THETN(I)
        SoilWater(I) = THETN(I)*DELX(I)
      end do
      
      
   END SUBROUTINE Leaching
      

end module Hydrolgy
    