module PRZM_CORE
   implicit none
   contains
   
   SUBROUTINE PRZM
   use  constants_and_Variables, ONLY: NAPP,NC,MXCPD, NCMPTS,WIND,temp,ITFLAG,spt,CAM,            & 
    THRUFL,ubt,really_not_thrufl,COVER,HEIGHT,ncrop, USLEC,MNGN,N1,cfac,                          & 
    THETN,BD,runof,thetas,ainf,juslec,nuslec,erflag,ncom2,IRFLAG,sedl,KD,pestr,spestr,pestr,SOILAP,ENPY,DAIR,KH,    & 
    OKH,HENRYK,QFAC, ELTERM,PLNTAP,NCHEM,CONDUC,ELTT,FAM, X,NPLOTS,                                & 
    header,current_year, CN_index, application_date,under_canopy_irrig,over_canopy_irrig,julday1900, &
    use_usleyears,NAPS,FLAG1,k_freundlich
   
   
   Use m_Wind
   Use m_Canopy
!   use readinputs
   use Pesticide_Applications  
   use Temperatue_Calcs
!   use EXAMS_file
   use Output_File
   use Hydrolgy
   use evapotranspiration
   use erosion
   use irrigation
   use plantgrowth
   use utilities
   use freundlich_isotherm
   
  ! use oldPRZM3
   implicit none
   
   INTEGER,save :: SRNFG =1       !starting run flag

   real     :: CNDBDY(3)     

   INTEGER      :: I,K
   REAL         :: ATEMP(2),PWIND(2),OLDKH(NCMPTS),ZCH,TOTCR
   Real         :: z0, d, urh, uch

   integer      :: julday  !this is the day of the year starting with Jan 1

   
   !******************************************
   CALL PLGROW(julday1900) !PLANT GROWTH 

   AINF(1)= 0.0
   THRUFL = 0.0
   really_not_thrufl = .FALSE.
  
   !**********IRRIGATION ************************ 
   
   IF (IRFLAG ==1) then                         !irrigate any time
      CALL IRRIG_PRZM5
   else IF (IRFLAG ==2 .AND. cover > 0.0) then  !irrigate if there is a crop
      CALL IRRIG_PRZM5
   else                                         !no irrigation
      under_canopy_irrig = 0.0
      over_canopy_irrig = 0.0
   end if

   !Find out if there is a change in the curve number or erosion factors
   !Simplified routine. Now does not depend on order of erorsion factors.  And wont get stick if first date is not emergence
  
   
   !this version of julian day is needed for the curve number and erosion routines
   julday = julday1900 -jd(current_year,1,1) +1
   
   
   select case (use_usleyears)
   case (1)
      do i = 1, nuslec(ncrop)
       if (julday1900  ==juslec(ncrop, i))then
  
           cfac=USLEC(NCROP,i)
           n1 = MNGN(NCROP,i)
           CN_index = i  !this is used for curve number
           exit
       end if
      end do      
       
   case default
      do i = 1, nuslec(ncrop)
       if (julday ==juslec(ncrop, i))then
  
           cfac=USLEC(NCROP,i)
           n1 = MNGN(NCROP,i)
           CN_index = i  !this is used for curve number
           exit
       end if
      end do

   end select


   
   
Call Runoff_PRZM5  
   

   CALL EVPOTR                  !EVAPORATION 
   CALL Leaching
   

   !*** EROSION *********************************
   SEDL= 0.0
   ELTT= 0.0
   IF (RUNOF .GT. 0.0 .AND. ERFLAG > 1) THEN
     CALL EROSN(JULDAY)  !calc loss of chem due to erosion
   END IF

   !******Pesticide Simulation ********************
   
   do K=1, NCHEM
       
       if (FLAG1) then
           Call Freundlich(k)  !In development, needs QA
       else
           KD =  k_freundlich
       end if 
       

       
       !See how Kd from the top compartment is the KD to be used for erosion
       ELTERM(K) = ELTT*KD(K,1)    !the ELTT variable has delx(1) inmplicitly in it  (NEEDS FIXING!  USE BANDAID FOR NOW, see later)
      


       
       DO I=1,NCOM2
         SOILAP(K,I) = 0.0
       end do

       IF (ITFLAG .EQ. 1) THEN
           IF (K .EQ. 1) CALL SLTEMP()
           IF (QFAC.GT.0.0) CALL Q10DK
          
           CALL KHCORR (SPT,HENRYK(K),ENPY(K),NCOM2,OLDKH)
           do I=1, NCOM2
               KH(K,I) = OLDKH(I)
           end do     
       ENDIF

       PLNTAP(K) = 0.0
  

    !check to see if this is an application day
    do i = 1,  NAPS
        if (application_date(i)==julday1900) CALL PESTAP(K,i)
    end do

     IF (FAM.EQ.2) CALL plant_pesticide(K)  ! fam=2 signifies that some applications were foliar   
 
    !*****************  NEEDS TO BE PUT IN A PROCEDURE ********************************
       ! CNDBDY: Boundary Layer's Conductance (cm day^-1) = 1/Rdb
       ! Rdb:    Boundary layer resistance (day/cm) = d / Dair,
       ! d:      Height of the stagnant air layer above the soil
       ! Dair:   Molecular Diffusivity in air (cm^2 day^-1)
       CNDBDY(K) = DAIR(K) / Height_stagnant_air_layer_cm

       ! CONDUC: Canopy Conductance Including Boundary Layer's Conductance (cm day^-1)
       CONDUC(K) = CNDBDY(K)

       ! When canopy develops, resistance type approach is used to estimate the volatilization             
       ! flux and concentration retains in the canopy

       ! HEIGHT: Canopy height (cm)


       IF (HEIGHT .GT. Minimum_Canopy_Height_cm) THEN
           ZCH = HEIGHT/100.0     ! convert to meter
           IF (ITFLAG .EQ. 0) THEN
                ATEMP(1)= 15.0
           ELSE
                ATEMP(1)= UBT
           ENDIF
           ATEMP(2)= TEMP

           ! Let u_2 and u_1 be wind speeds measured at
           ! heights z_2 and z_1 respectively. Then
           !
           !        u_2                    u_1
           ! ------------------  =  ------------------
           ! Ln((z_2-d_2)/z0_2)     Ln((z_1-d_1)/z0_1)
           !
           ! where
           !     u_i : wind speed at height z_i
           !     z_i : height at which the measurement was taken (m)
           !     d_i : zero plane displacement (m)
           !     z0_i: surface roughness length or roughness height (m)
           !
           ! This equation assumes the atmosphere is neutrally stable,
           ! i.e., phi_m = 1, which implies psi_m = 0.
           !
           ! Given the wind speed at reference conditions (urh), compute
           ! the wind speed at the top of the canopy (uch). Assume the
           ! atmosphere is neutrally stable.
           !
           ! The wind speed at reference conditions (urh) is retrieved from the
           ! metereological file. The aerodynamic parameters for wind speed
           ! computations are set by the subroutine Get_Aerodynamic_Parameters.
           ! In the absence of przm input file values, the routine assumes
           ! the conditions of the meteorological Daily Values File (*.dvf),
           ! i.e., Open Flat Terrain (used for Metereological Stations), and
           ! wind measurements normalized to 10 meters.
           ! See subroutines IniVar and Get_Aerodynamic_Parameters.
           !   Wind_Reference_Height = 10.0
           !   Wind_z0 = 0.03
           !   Wind_D  = 0.0

           ! Computes zero displacement height, D (meter)
           ! and the roughness length, Z0 (meter)
           Call Get_Crop_Params (zch, z0, d)

           ! urh: wind speed (meter/day) at reference height.
           !      units of WIND are cm/sec.
           !      1 cm/sec is equivalent to 864.0 meter/day
           !      subroutine canopy expects wind speed in meter/day
           ! uch: wind speed at the top of the canopy (zch)

           urh = WIND * 864.0
           uch = urh * Log((zch-d)/z0) /Log((uWind_Reference_Height-uWind_D)/uWind_z0)

           PWIND(1)= 0.0
           PWIND(2)= uch

           IF(HENRYK(K).GT.0.0.AND.URH.GT.0.0)THEN
               CALL Canopy(ATEMP,PWIND,ZCH,TOTCR )
               !this puts crcnc (which is array of 2 canopy resistances) into module for use in output ??? 
               ! CNDBDY: Boundary Layer's Conductance (cm/day)
               ! CONDUC: Canopy Conductance Including Boundary Layer's Conductance (cm/day)
               ! TOTCR:  Total canopy resistance (cm/day)
               CONDUC(K) = 1.0 / (1.0/CNDBDY(K) + TOTCR)
           ELSE
               TOTCR=0.0
           ENDIF
       ENDIF
       !***************************************************************************************
       call PRZM5_tridiagsetup(K)
       
       do I=1,NCOM2
          SPESTR(K,I)=X(I)    !X is output from tridiagonal
          ! store SPESTR for this zone (for use w/ MASCOR)
          PESTR(K,I)=SPESTR(K,I)*(THETN(I)+KD(K,I)*BD(I)+(THETAS(I)-THETN(I))*KH(K,I))/THETN(I)
       end do
                          
       IF (NPLOTS .GT. 0 .AND. K .EQ. NCHEM) THEN
          HEADER = HEADER + 1
          IF (HEADER .EQ. 1) SRNFG = 1
          CALL OUTTSR(SRNFG,HEIGHT)    
       ENDIF


       SRNFG = 0
       IF (ITFLAG .EQ. 1) THEN
          do  I=1,NCOM2
            OKH(K,I) = KH(K,I)
          end do
       ENDIF
   end do
                           
   END Subroutine PRZM
   !***************************************************************************
   
   
   


  !********************************************************************************************* 
  SUBROUTINE Q10DK
      !converts degradation rate by Q10
      use  constants_and_Variables, ONLY: NAPP,NC, NCMPTS,spt,DELX,ncom2,DWRAT1,DSRAT1,DGRAT1,DWRATE,DSRATE,DGRATE,QFAC,TBASE,  &
      NCHEM,NHORIZ,THKNS
      implicit none
      REAL         TTHKNS,MODFC,T,Q10FAC
      INTEGER      J,JB,IB,IBM1,L
  
      do L=1,NCHEM
!     	  assign horizon soil profile values to individual soil layers
      
           IB = NHORIZ
           T  = 0.0
           TTHKNS = THKNS(IB)
           do J = 1, NCOM2-1
             IBM1= IB - 1
             JB  = NCOM2 - J + 1
             T   = T + DELX(JB)
             MODFC  = 0.0
             Q10FAC=QFAC**((SPT(JB)-TBASE)/10.)
             IF (T .LE. TTHKNS) THEN
               DWRATE(L,JB) = DWRAT1(L,IB)*Q10FAC
               DSRATE(L,JB) = DSRAT1(L,IB)*Q10FAC
               DGRATE(L,JB) = DGRAT1(L,IB)*Q10FAC
             ELSE
               MODFC=(T-TTHKNS)/DELX(JB)
               DWRATE(L,JB)=((DWRAT1(L,IB)*(1.-MODFC))+(DWRAT1(L,IBM1)*MODFC))*Q10FAC
               DSRATE(L,JB)=((DSRAT1(L,IB)*(1.-MODFC))+(DSRAT1(L,IBM1)*MODFC))*Q10FAC
               DGRATE(L,JB)=((DGRAT1(L,IB)*(1.-MODFC))+(DGRAT1(L,IBM1)*MODFC))*Q10FAC
               IB=IB-1
               TTHKNS=TTHKNS+THKNS(IB)
             ENDIF
           end do
           DWRATE(L,1) = DWRAT1(L,1)*Q10FAC
           DSRATE(L,1) = DSRAT1(L,1)*Q10FAC
           DGRATE(L,1) = DGRAT1(L,1)*Q10FAC
      end do

  END SUBROUTINE Q10DK
  
   
   !*********************************************************************************************************************
  
  SUBROUTINE PRZM5_tridiagsetup(K)

    use  constants_and_Variables, ONLY: NAPP,NC,MXCPD, NCMPTS,NCOMRZ,SoilWater,et,THETN,DELX,THETO,BD,runof,thetas,         &
      RNCMPT,ncom2,vel,PVFLUX,ADFLUX,DFFLUX,SDKFLX, SUPFLX, UPFLUX, DWRATE,DSRATE,DGRATE,KD,spestr,gamma1, &
      DISP,DKRW12,DKRW13,DKRW23,DAIR,OKH,KH,soil_applied_washoff,ELTERM,CONDUC,DKFLUX, &
      ERFLUX, WOFLUX,ROFLUX,RZFLUX,TRFLUX,SRCFLX, &
      TTRFLX,TSRCFX,X,delt,DCOFLX,runoff_intensity, erosion_intensity,erosion_compt, model_mg_ro,data_date,model_mg_er,julday1900
      
    
    use utilities
      implicit none

!     Sets up the coefficient matrix for the solution of the soil pesticide transport equation. It then calls an equation
!     solver for the tridiagonal matrix and sets up pesticide flux terms using the new concentrations.
!     Modification date: 2/18/92 JAM
!     Further modified at AQUA TERRA Consultants to hard code the
!     pesticide extraction depth to 1 cm. 9/93

      INTEGER     K

!     K      - chemical number being simulated (1-3)
!     CMPT   - number of compartments which make up the pesticide runoff extraction depth

      INTEGER      I,J

      REAL         THAIR(NCMPTS),DGAIR(NCMPTS)
      REAL         VTERM

      real ::   A(NCMPTS),B(NCMPTS),C(NCMPTS),F(NCMPTS)
      !real ::  RFFLUX(3,NCMPTS)
      integer :: ncom2m
  
      
      NCOM2M= NCOM2- 1
      
      do J=1,NCOM2
        SRCFLX(1,J)=0.0
        THAIR(J)=THETAS(J)-THETN(J)
        IF (THAIR(J) .LT. 0.0) THAIR(J) = 0.0
!       DGAIR now includes correction for air to bulk volume
        DGAIR(J)=(THAIR(J)**(10./3)/THETAS(J)**2)*DAIR(K) * THAIR(J)
      end do
     
!     Set up coefficients for surface layer
      ! Set up tridiagonal system
      ! a(i) y(i-1)  +  b(i) y(i)  +  c(i) y(i+1) = f(i)
      ! a(1) = c(n) = 0

      J=1

      A(1)= 0.0
      B(1)= ((DISP(K,1)*THETN(1)+KH(K,1)*DGAIR(1))/(DELX(1)* DELX(1))+VEL(1)*THETN(1)/DELX(1)    &
             +(DWRATE(K,1)*THETN(1))+(DSRATE(K,1)*KD(K,1)*BD(1))          &
             +(DGRATE(K,1)*THAIR(1)*KH(K,1))    &
             +GAMMA1(K,1)*ET(1)*THETN(1)/SoilWater(1)  ) *DELT                  &
      
             +THETN(1) + KD(K,1)*BD(1) + THAIR(1)*KH(K,1)+CONDUC(K)*KH(K,1)*DELT/DELX(1)
       
  
      
      B(1) = B(1) + RUNOF*runoff_intensity(1)       !*delx(1)
      B(1) = B(1) + ELTERM(K)*erosion_intensity(1)*delx(1)  !*delx(1)/delx(1)  !first delx fixes the elterm problem
      C(1)= -(DISP(K,2)*THETN(2)+KH(K,1)*DGAIR(2))*DELT/(DELX(1)* 0.5*(DELX(1)+DELX(2)))
      
      F(1)= (THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*OKH(K,1))*SPESTR(K,1) +  &
             (soil_applied_washoff(K,1)*DELT/DELX(1))+SRCFLX(K,1)/DELX(1)*DELT 

!     Calculate coefficient of non-boundary soil layers

     

      do I=2,NCOM2M
        A(I)= (-(DISP(K,I-1)*THETN(I-1)+KH(K,I-1)*DGAIR(I-1))/(DELX(I)*0.5*(DELX(I-1)+DELX(I)))-VEL(I-1)*THETN(I-1)/DELX(I))*DELT
        B(I)= ((DISP(K,I)*THETN(I)+KH(K,I)*DGAIR(I))/(DELX(I)*0.5*(DELX(I-1)+DELX(I)))                  &
              + (DISP(K,I)*THETN(I)+KH(K,I)*DGAIR(I))/(DELX(I)*0.5*(DELX(I)+DELX(I+1)))                 &
              +VEL(I)*THETN(I)/DELX(I)                          &
              +(DWRATE(K,I) *THETN(I))                          &
              +(DSRATE(K,I) *KD(K,I)*BD(I))                     &
              +(DGRATE(K,I) *THAIR(I)*KH(K,I))                  &
              +GAMMA1(K,I)*ET(I)*THETN(I)/SoilWater(I))*DELT+THETN(I)+KD(K,I)*BD(I)+THAIR(I)*KH(K,I)
        
         if (i<=rncmpt)        B(I)=B(I)+ RUNOF*runoff_intensity(i)              !*delx(i)
         
         !YES the below is correct, below delx(1) accounts for dividing & multiplying by delx(i)
         if (i<=erosion_compt) B(I)=B(I)+ ELTERM(k)*erosion_intensity(i)*delx(1) 
         
         C(I)= -(DISP(K,I+1)*THETN(I+1)+KH(K,I+1)*DGAIR(I+1))*DELT/(DELX(I)*0.5*(DELX(I)+DELX(I+1)))
         
         F(I)=(THETO(I)+KD(K,I)*BD(I)+(THETAS(I)-THETO(I))*OKH(K,I))*SPESTR(K,I)+  &
               (soil_applied_washoff(K,I)*DELT/DELX(I)) +SRCFLX(K,I)/DELX(I)*DELT

      end do

!     Calculate coefficients of bottom layer

      VTERM   = VEL(NCOM2) * THETN(NCOM2) / DELX(NCOM2)

      A(NCOM2)=(-(DISP(K,NCOM2M)*THETN(NCOM2M)+KH(K,NCOM2M)*DGAIR(NCOM2M))                 &
              /(DELX(NCOM2)*0.5*(DELX(NCOM2M)+DELX(NCOM2)))-VEL(NCOM2M)*THETN(NCOM2M)/DELX(NCOM2))*DELT
      
      B(NCOM2)= ((DISP(K,NCOM2)*THETN(NCOM2)+KH(K,NCOM2)*DGAIR(NCOM2))/(DELX(NCOM2)*DELX(NCOM2))+ VTERM      &
              +(DWRATE(K,NCOM2)*THETN(NCOM2))        &
              +(DSRATE(K,NCOM2)*KD(K,NCOM2)*BD(NCOM2))+DGRATE(K,NCOM2)*THAIR(NCOM2)*KH(K,NCOM2))*DELT       &
              +THETN(NCOM2)+KD(K,NCOM2)*BD(NCOM2)+THAIR(NCOM2)*KH(K,NCOM2)

      C(NCOM2)= 0.0
      F(NCOM2)= (THETO(NCOM2)+KD(K,NCOM2)*BD(NCOM2)+(THETAS(NCOM2)-THETO(NCOM2))*OKH(K,NCOM2))*SPESTR(K,NCOM2)  &
               +(soil_applied_washoff(K,NCOM2)*DELT/DELX(NCOM2))+SRCFLX(K,NCOM2)/DELX(NCOM2)*DELT
   

      CALL TRDIAG (A,B,C,X,F,NCOM2)
            
!     Calculate pesticide fluxes

      ! PVFLUX: Daily Soil Pesticide Volatilization Flux (g cm^-2 day^-1)
      PVFLUX(K,1) = -CONDUC(K)*X(1)*KH(K,1)
      IF(ABS(PVFLUX(K,1)).LT.1.E-34)PVFLUX(K,1)=0.0
      DFFLUX(K,1)=DISP(K,1)/(0.5*(DELX(1)+DELX(2)))*X(1)*THETN(1)-DISP(K,2)/(0.5*(DELX(1)+DELX(2)))*X(2)*THETN(2)
      ADFLUX(K,1)=VEL(1)*X(1)*THETN(1)

      DKFLUX(K,1)=DELX(1)*X(1)*(  DWRATE(K,1)*THETN(1) +   DSRATE(K,1)*BD(1)*KD(K,1)+    DGRATE(K,1)*THAIR(1)*KH(K,1)   )

!*****************************************************************************     
!Note from Dirk Young October 7, 2010:  the following code appeared after version 3.2beta
!in an apparent attempt to standardize or correct a degradation issue by ORD/CEAM.  
!This code in combination with the code in Rsinp2.for (przmrd) causes a double
!counting of degradation.  This issue was fixed by removing code in routine przmrd
!See October 7, 2010 comment in that routine for additional information.



      if(k.eq.1)then
        trflux(1,1)=(dkrw12(1)+dkrw13(1))*dkflux(1,1)
        srcflx(1,1)=0.0
        srcflx(2,1)=dkrw12(1)*dkflux(1,1) 
        srcflx(3,1)=dkrw13(1)*dkflux(1,1)
        dkflux(1,1)=dkflux(1,1)-trflux(1,1)             
      elseif(k.eq.2)then
        trflux(2,1)=dkrw23(1)*dkflux(2,1)
        srcflx(3,1)=srcflx(3,1)+trflux(2,1)
        dkflux(2,1)=dkflux(2,1)-trflux(2,1)
      endif
!*******************************************************************



      ROFLUX(K) =  sum( RUNOF*runoff_intensity(1:RNCMPT)*X(1:RNCMPT) *DELX(1:RNCMPT) )
      ERFLUX(K) =  sum( ELTERM(K)*DELX(1)*X(1:erosion_compt)*erosion_intensity(1:erosion_compt)*DELX(1:erosion_compt) )
      

      

      do I=2,NCOM2M 
        PVFLUX(K,I)=DGAIR(I)*KH(K,I)/(0.5*(DELX(I)+DELX(I+1)))*X(I)-DGAIR(I+1)*KH(K,I+1)/(0.5*(DELX(I)+DELX(I+1)))*X(I+1)
        IF(ABS(PVFLUX(K,I)).LT.1.E-34)PVFLUX(K,I)=0.0
        DFFLUX(K,I)=DISP(K,I)/(0.5*(DELX(I)+DELX(I+1)))*THETN(I)*X(I)-DISP(K,I+1)/(0.5*(DELX(I)+DELX(I+1)))*THETN(I+1)*X(I+1)
        ADFLUX(K,I)=VEL(I)*X(I)*THETN(I)
            
        DKFLUX(K,I)=DELX(I)*X(I)*(DWRATE(K,I)*THETN(I)+DSRATE(K,I)*BD(I)*KD(K,I)+DGRATE(K,I)*THAIR(I)*KH(K,I)   )
        UPFLUX(K,I)=GAMMA1(K,I)*ET(I)*X(I)
        if(k.eq.1)then
            trflux(1,i)=(dkrw12(i)+dkrw13(i))*dkflux(1,i)
            srcflx(1,i)=0.0
            srcflx(2,i)=dkrw12(i)*dkflux(1,i)
            srcflx(3,i)=dkrw13(i)*dkflux(1,i)
            dkflux(1,i)=dkflux(1,i)-trflux(1,i)
        elseif(k.eq.2)then
           trflux(2,i)=dkrw23(i)*dkflux(2,i)
           srcflx(3,i)=srcflx(3,i)+trflux(2,i)
           dkflux(2,i)=dkflux(2,i)-trflux(2,i)
        endif
      end do

      RZFLUX(K)= DISP(K,NCOMRZ)/(0.5*(DELX(NCOMRZ)+DELX(NCOMRZ+1)))*THETN(NCOMRZ)*X(NCOMRZ)-DISP(K,NCOMRZ+1)/(0.5*(DELX(NCOMRZ+1) &
       +DELX(NCOMRZ)))*THETN(NCOMRZ+1)*X(NCOMRZ+1)+(VEL(NCOMRZ)*X(NCOMRZ)*THETN(NCOMRZ))
      

      DFFLUX(K,NCOM2)=0.
      PVFLUX(K,NCOM2)=0.
      UPFLUX(K,NCOM2)=0.
      ADFLUX(K,NCOM2) = VTERM * DELX(NCOM2) * X(NCOM2)
      
      
      DKFLUX(K,NCOM2)=DELX(NCOM2)*X(NCOM2)*(DWRATE(K,NCOM2)*THETN(NCOM2)+DSRATE(K,NCOM2)*BD(NCOM2)*KD(K,NCOM2)   &
                 +DGRATE(K,NCOM2)*THAIR(NCOM2)*KH(K,NCOM2)   )
      if(k.eq.1)then
        trflux(1,ncom2)=(dkrw12(ncom2)+dkrw13(ncom2))*dkflux(1,ncom2)
        srcflx(1,ncom2)=0.0
        srcflx(2,ncom2)=dkrw12(ncom2)*dkflux(1,ncom2)
        srcflx(3,ncom2)=dkrw13(ncom2)*dkflux(1,ncom2)
        dkflux(1,ncom2)=dkflux(1,ncom2)-trflux(1,ncom2)
      elseif(k.eq.2)then
        trflux(2,ncom2)=dkrw23(ncom2)*dkflux(2,ncom2)
        srcflx(3,ncom2)=srcflx(3,ncom2)+trflux(2,ncom2)
        dkflux(2,ncom2)=dkflux(2,ncom2)-trflux(2,ncom2)
      endif

!     Calculate core flux values.
!     Multiply internal units of GR/CM**2 by 10**5 so output
!     is expressed in units of KG/HA (as in the input).
!
      DCOFLX(K) = DISP(K,NCOM2)/DELX(NCOM2)*THETN(NCOM2)*X(NCOM2)- DISP(K,NCOM2)/DELX(NCOM2)*THETN(NCOM2)*X(NCOM2)   &
               + (VEL(NCOM2)*X(NCOM2)*THETN(NCOM2))
      DCOFLX(K) = DCOFLX(K) * 1.0E5

!     Accumulate fluxes from soil layers for output

      WOFLUX(K)= 0.0

      SUPFLX(K)= 0.0
      SDKFLX(K)= 0.0
      TTRFLX(K)= 0.0
      TSRCFX(K)= 0.0

      do I=1,NCOM2
         WOFLUX(K)= WOFLUX(K)+soil_applied_washoff(K,I)
         SDKFLX(K)= SDKFLX(K)+DKFLUX(K,I)
         SUPFLX(K)= SUPFLX(K)+UPFLUX(K,I)
         TTRFLX(K)= TTRFLX(K)+TRFLUX(K,I)
         TSRCFX(K)= TSRCFX(K)+SRCFLX(K,I)
      end do


      
      !the folowing are captured for calibration studies, not needed otherwise
     where (data_date == julday1900) model_mg_ro =  ROFLUX(K)
     where (data_date == julday1900) model_mg_er =  ERFLUX(K)  

      

     

         

      END SUBROUTINE PRZM5_tridiagsetup

  
  !******************************************************************************************
  
  
  
  
  
  
  
  
  
end module PRZM_CORE
   
