module Pesticide_Applications
implicit none
contains
    
SUBROUTINE PESTAP(K, NAPPC)
      use  constants_and_Variables, ONLY: NAPP,NC,MXCPD,NCMPTS,COVER,WEIGHT,ncrop,IFSCND,CAM,  &
         DELX,THETO,BD,thetas,DEPI,KD,pestr,spestr,TAPP,appeff,FILTRA,SOILAP,FOLPST,KH,PLNTAP,Tband_top
      use utilities
      implicit none
      
    !K is the chenmical ID Number
      ! Computes amount and location of pesticide application (foliage, soil surface, or soil layer)

      !NOTE: IFSCND is set to 0 after each foliar application, but left untouched after ground application, Why?
      
      
      INTEGER,intent(in) ::   K
      
      REAL  :: DMAX,APPAMT,SLOPE,BASE
      integer:: CAM_local 
      integer,intent(in)::  nappc
      
 
      
      CAM_local = CAM(K,NAPPC)
      
      select case (CAM_local)
      case (1)  !Soil surface pesticide application
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
        !Solve for base of isosceles triangle given: length of 1 leg and area = 1.0
        DMAX = 4.0
        BASE=2./DMAX
        SLOPE=(-BASE/DMAX) 
        call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K) 
  
      case (2) ! Linear foliar pesticide application
        IF (DELX(1) .GE. 2.0) THEN !if the del x is greater than 2 cm, just distribute it in the top comnpartment
          PLNTAP(K)    = COVER * TAPP(K,NAPPC) * APPEFF(K,NAPPC)
          SOILAP(K,1)  = (1.0- COVER) * TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          PLNTAP(K)    = COVER * TAPP(K,NAPPC) * APPEFF(K,NAPPC)
          APPAMT       = (1.0- COVER) * TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          !Solve for base of isosceles triangle given: length of 1 leg and area = 1.0   
          DMAX = 4.0
          BASE=2./DMAX
          SLOPE=(-BASE/DMAX)
          call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K)
        ENDIF
        
        FOLPST(K)    = FOLPST(K)+ PLNTAP(K)
        IFSCND(K,NCROP)= 0

      case (3) !Exponential foliar pesticide application
        IF (DELX(1) .GE. 2.0) THEN
          PLNTAP(K)    = (1.0-EXPCHK(-FILTRA*WEIGHT)) *(TAPP(K,NAPPC)*APPEFF(K,NAPPC))
          SOILAP(K,1)  = (TAPP(K,NAPPC)*APPEFF(K,NAPPC))- PLNTAP(K)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1) +(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          PLNTAP(K)=(1.0-EXPCHK(-FILTRA*WEIGHT))*(TAPP(K,NAPPC)*APPEFF(K,NAPPC))
          APPAMT  = (TAPP(K,NAPPC)*APPEFF(K,NAPPC))- PLNTAP(K)
          !Solve for base of isosceles triangle given:length of 1 leg and area = 1.0    
          DMAX = 4.0
          BASE=2./DMAX
          SLOPE=(-BASE/DMAX)
          call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K) 
        ENDIF

        FOLPST(K)    = FOLPST(K)+ PLNTAP(K)
        IFSCND(K,NCROP)= 0  
        
      case (4) !Uniform  Incorporated pesticide application method 1
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
        DMAX = DEPI(K,NAPPC)
        IF(DEPI(K,NAPPC).EQ.0.0)THEN
          SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1) +(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          call pesticide_uniform_distribution(APPAMT,DMAX,K) 
        ENDIF
        
      case (5) ! Incorporated increasing with depth pesticide application 
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
!       Solve for base of isosceles triangle given:length of 1 leg and area = 1.0
        DMAX = DEPI(K,NAPPC)
        BASE=2./DMAX
        SLOPE=(-BASE/DMAX)
        IF(DEPI(K,NAPPC) == 0.0)THEN
          SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          CALL pesticide_increasing_distribution(APPAMT,DMAX,BASE,SLOPE,K)
        ENDIF

      case (6) !Incorporated pesticide application method 3
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
        !  Solve for base of isosceles triangle given: length of 1 leg and area = 1.0
        DMAX = DEPI(K,NAPPC)
        BASE=2./DMAX
        SLOPE=(-BASE/DMAX)
        IF(DEPI(K,NAPPC).EQ.0.0)THEN
          SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K) 
        ENDIF
        
      case (7)
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
        DMAX = DEPI(K,NAPPC)
        IF(DEPI(K,NAPPC).EQ.0.0)THEN
          SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          CALL pesticide_Tband_distribution(APPAMT,DMAX,K, Tband_top(k,NAPPC))
        ENDIF
        
      case(8)
        PLNTAP(K)= 0.
        APPAMT = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
        !Solve for base of isosceles triangle given:length of 1 leg and area = 1.0
        DMAX = DEPI(K,NAPPC)
        BASE=0.
        SLOPE=0.
        IF(DEPI(K,NAPPC).EQ.0.0)THEN
          SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          CALL pesticide_atdepth_distribution(APPAMT,DMAX,K)
        ENDIF
        
      case (9)
        IF (DELX(1) .GE. 2.0) THEN
          PLNTAP(K)    = COVER * TAPP(K,NAPPC) * APPEFF(K,NAPPC)
          SOILAP(K,1)  = (1.0- COVER) * TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1) +(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          PLNTAP(K)    = COVER * TAPP(K,NAPPC) * APPEFF(K,NAPPC)
          APPAMT       = (1.0- COVER) * TAPP(K,NAPPC)*APPEFF(K,NAPPC)
          !Solve for base of isosceles triangle given:length of 1 leg and area = 1.0     
          IF(DEPI(K,NAPPC).GT.0.0)THEN
             DMAX = DEPI(K,NAPPC)
          ELSE
             DMAX = 4.0
          ENDIF
          BASE=2./DMAX
          SLOPE=(-BASE/DMAX)
          IF(DEPI(K,NAPPC).EQ.0.0)THEN
            SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
            PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
            SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
          ELSE
            call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K)
          ENDIF
        ENDIF
        FOLPST(K)    = FOLPST(K)+ PLNTAP(K)
        IFSCND(K,NCROP)= 0

      case (10) !Exponential foliar pesticide application
        IF (DELX(1) .GE. 2.0) THEN
          PLNTAP(K)    = (1.0-EXPCHK(-FILTRA*WEIGHT)) *(TAPP(K,NAPPC)*APPEFF(K,NAPPC))
          SOILAP(K,1)  = (TAPP(K,NAPPC)*APPEFF(K,NAPPC))- PLNTAP(K)
          PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
          SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
        ELSE
          PLNTAP(K)=(1.0-EXPCHK(-FILTRA*WEIGHT)) *(TAPP(K,NAPPC)*APPEFF(K,NAPPC))
          APPAMT  = (TAPP(K,NAPPC)*APPEFF(K,NAPPC))- PLNTAP(K)

          !Solve for base of isosceles triangle given:length of 1 leg and area = 1.0      
          IF(DEPI(K,NAPPC).GT.0.0)THEN
             DMAX = DEPI(K,NAPPC)
          ELSE
             DMAX = 4.0
          ENDIF
          BASE=2./DMAX
          SLOPE=(-BASE/DMAX)

          IF(DEPI(K,NAPPC).EQ.0.0)THEN
            SOILAP(K,1)  = TAPP(K,NAPPC)*APPEFF(K,NAPPC)
            PESTR(K,1)   = PESTR(K,1)+ SOILAP(K,1)/(DELX(1)*THETO(1))
            SPESTR(K,1)  = PESTR(K,1)*THETO(1)/(THETO(1)+KD(K,1)*BD(1)+(THETAS(1)-THETO(1))*KH(K,1))
          ELSE
            call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K)
          ENDIF
        ENDIF
        FOLPST(K)    = FOLPST(K)+ PLNTAP(K)
        IFSCND(K,NCROP)= 0
      end select
      
   
      
           
END SUBROUTINE PESTAP

    
!******************************************************************************************************************************
SUBROUTINE plant_pesticide(K)
      use  constants_and_Variables, ONLY: NAPP,NC,MXCPD,NCMPTS,ncrop,IFSCND,THETO,DELX,thetas,&
        RNCMPT,DEPI,FEXTRC,PLDKRT,SOILAP,PTRN12,PTRN13,PTRN23,FPDLOS,FOLPST,PLVKRT,FPVLOS,FPLOS12,FPLOS13,FPLOS23,FOLP0, &
        soil_applied_washoff,NCHEM,CRPAPP,delt,canopy_flow
        
      use utilities
      implicit none

      !Determines amount of pesticide which disappears from plant surface by first order decay.  The variable PLDKRT
      !is a pseudo first order decay rate which may include processes of volatilization, oxidation, photolysis, etc.
      !also determines pesticide washed off during rainfall events.

      INTEGER,intent(in) :: K

      REAL         WFRC(NCMPTS)
      REAL         APPAMT,DMAX,SLOPE,BASE,WFRCTT,FOLP1
      REAL         TERM,TERM1,TERM2,TERM3,TERM4,TERM5
      REAL          FPWLOS,ALIMIT
      REAL         PFRC1,PFRC2,PFRC3,PFRC4,PFRC5
      INTEGER      I

      DATA ALIMIT /1.0E-30/

  
   
      
      IF (IFSCND(K,NCROP) .GE. 4) THEN
         !Signal to bypass this routine
      ELSE IF (IFSCND(K,NCROP) .EQ. 0 .OR. IFSCND(K,NCROP) .EQ. 3) THEN
           !Before harvest or after harvest under surface-residue condition
           
           TERM   = EXPCHK(-(FEXTRC(K)*canopy_flow))   !chnaged to canopy_flow from precip  by dfy 2012
           IF (canopy_flow .LE. 0.0) TERM= 1.0
           
           FOLP0(K) = FOLPST(K)
           FPWLOS   = FOLP0(K)*(1.0-TERM)
           FOLP1    = FOLPST(K)-FPWLOS
           
           ! PLDKRT:	pesticide decay rate constant on plant foliage (days^-1).
           ! PLVKRT: pesticide volatilization decay rate constant on plant foliage (days^-1)
         IF(NCHEM.EQ.1)THEN
             TERM1    = (EXPCHK((-PLDKRT(K)*DELT)))
             TERM2    = (EXPCHK((-PLVKRT(K)*DELT)))
             IF(((1-TERM1)+(1-TERM2)).GE.1.)THEN
               PFRC1=(1.-TERM1)/((1.-TERM1)+(1.-TERM2))
               PFRC2=(1.-TERM2)/((1.-TERM1)+(1.-TERM2))
             ELSE
               PFRC1=1.
               PFRC2=1.
             ENDIF
         ELSEIF(NCHEM.GT.1)THEN
            TERM1    = EXPCHK(-PLDKRT(K)*DELT)
            TERM2    = EXPCHK(-PLVKRT(K)*DELT)
            TERM3    = EXPCHK(-PTRN12*DELT)
            TERM4    = EXPCHK(-PTRN13*DELT)
            TERM5    = EXPCHK(-PTRN23*DELT)
            IF(K.EQ.1)THEN
                IF(((1-TERM1)+(1-TERM2)+(1-TERM3)+(1-TERM4)).GE.1.)THEN
                   PFRC1=(1.-TERM1)/((1.-TERM1)+(1.-TERM2)+(1.-TERM3)+(1.-TERM4))
                   PFRC2=(1.-TERM2)/((1.-TERM1)+(1.-TERM2)+(1.-TERM3)+(1.-TERM4))
                   PFRC3=(1.-TERM3)/((1.-TERM1)+(1.-TERM2)+(1.-TERM3)+(1.-TERM4))
                   PFRC4=(1.-TERM4)/((1.-TERM1)+(1.-TERM2)+(1.-TERM3)+(1.-TERM4))
                ELSE
                   PFRC1=1.
                   PFRC2=1.
                   PFRC3=1.
                   PFRC4=1.
                ENDIF
                FPLOS12= PFRC3*(FOLP1*(1.0-TERM3))
                FPLOS13= PFRC4*(FOLP1*(1.0-TERM4))
            ELSEIF(K.EQ.2)THEN
              IF(((1-TERM1)+(1-TERM2)+(1-TERM5)).GE.1.)THEN
                PFRC1=(1.-TERM1)/((1.-TERM1)+(1.-TERM2)+(1.-TERM5))
                PFRC2=(1.-TERM2)/((1.-TERM1)+(1.-TERM2)+(1.-TERM5))
                PFRC5=(1.-TERM5)/((1.-TERM1)+(1.-TERM2)+(1.-TERM5))
              ELSE
                PFRC1=1.
                PFRC2=1.
                PFRC5=1.
              ENDIF 
              FPLOS23= PFRC5*(FOLP1*(1.0-TERM5))
            ENDIF  
         ENDIF
         ! FPDLOS: Current Daily Foliar	Pesticide Decay Loss	(g cm^-2)
         ! FPVLOS: Daily Foliage Pesticide	Volatilization Flux(g cm^-2)
         FPDLOS(K)= PFRC1*(FOLP1*(1.0-TERM1))
         FPVLOS(K)= PFRC2*(FOLP1*(1.0-TERM2))


         
         IF(NCHEM.EQ.1)THEN
             FOLPST(K)=FOLPST(K)-FPDLOS(K)-FPWLOS-FPVLOS(K)
             
           ELSEIF(NCHEM.GT.1)THEN
             IF(K.EQ.1)THEN
               FOLPST(K)=FOLPST(K)-FPDLOS(K)-FPWLOS-FPVLOS(K)-FPLOS12-FPLOS13
             ELSEIF(K.EQ.2)THEN
               FOLPST(K)=FOLPST(K)+FPLOS12-FPDLOS(K)-FPWLOS-FPVLOS(K)-FPLOS23
             ELSEIF(K.EQ.3)THEN
               FOLPST(K)=FOLPST(K)+FPLOS13+FPLOS23-FPDLOS(K)-FPWLOS-FPVLOS(K)
             ENDIF
         ENDIF
         ! Check for underflow
         IF(FOLPST(K).LE.ALIMIT) FOLPST(K)=0.0
         do I=1,RNCMPT
           soil_applied_washoff(K,I) = 0.0
         end do
         
         WFRCTT=0.0
           
        !CHECK THIS OUT TO FIND OUT WHAT IT IS DOING  ----dfy
        
        !Pesticide is distributed in proportion to the pore volume of the compartments
        !This is previously undocumented.
        IF (canopy_flow .GT. 0.0) THEN
            do I=1,RNCMPT
               WFRC(I)=(THETAS(I)-THETO(I))*DELX(I)
               WFRCTT=WFRCTT+WFRC(I)
            end do
            do I=1,RNCMPT
               soil_applied_washoff(K,I) = (WFRC(I)/WFRCTT)*FPWLOS  !pesticide applied by washoff
            end do
        ENDIF          
      ELSE IF (IFSCND(K,NCROP) .EQ. 1) THEN !HARVEST APPLICATION of Pesticide from foliage
        ! Solve for base of isosceles triangle given:length of 1 leg and area = 1.0
        CRPAPP(K)=1
        !IF(DEPI(K,NAPPC).GT.0.0)THEN
        !   DMAX = DEPI(K,NAPPC)
        !ELSE
        !   DMAX = 2.0
        !ENDIF
        
        DMAX = 4.0
        BASE=2./DMAX
        SLOPE=(-BASE/DMAX)
        APPAMT=FOLPST(K)
 
        !pesticide mass additions fro the tridiag are set in the pdstrb routine
        !there is an issue that the CAM used is the CAM of the future application
         !CALL PDSTRB(APPAMT,DMAX,BASE,SLOPE,K)
         
         !Now independent of CAM
         call pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,K) 
               
         !Seems like Double counting of pesticide from foliage,eliminated by dfy Feb 2013
         !actually not used though, 
         !SOILAP(K,1)= FOLPST(K)
      
         IFSCND(K,NCROP) = 4
         FPVLOS(K)  = 0.0
         FPDLOS(K)  = 0.0
         FOLPST(K)  = 0.0
         FOLP0(K)   = 0.0
         !do I=1,RNCMPT
         !  soil_applied_washoff(K,I) = 0.0
         !end do
         soil_applied_washoff(K,:) = 0.
                     
      ELSE IF (IFSCND(K,NCROP) .EQ. 2) THEN
!        After harvest remove all pesticide from canopy
         
         IFSCND(K,NCROP) = 4
         FPVLOS(K) = 0.0
         FPDLOS(K) = 0.0
         FOLPST(K) = 0.0
         FOLP0(K)  = 0.0
         !do I=1,RNCMPT
         !  soil_applied_washoff(K,I) = 0.0
         !end do
         soil_applied_washoff(K,:) = 0.0
      ENDIF
      
      
      
END SUBROUTINE plant_pesticide


!******************************************************************************************************************************
SUBROUTINE pesticide_decreasing_distribution(APPAMT,DMAX,BASE,SLOPE,CHEM)
      use  constants_and_Variables, ONLY: NAPP,NC,NCMPTS,DELX,THETO,BD,thetas,KD,pestr,spestr,SOILAP,KH
      implicit none
      ! This routine distributes a chemical application down to an input depth decreasing proportionally with depth.
      ! Used for CAM 1 and 6 and Harvest
      
      ! APPAMT - total amount of chemical to distribute in the soil
      ! CHEM   - chemical id number (1-3)
      ! DMAX   - depth to which chemical should be applied;
      ! BASE   - initial starting amount of application
      ! SLOPE  - slope of linearly decreasing application

      INTEGER  CHEM
      REAL     APPAMT,DMAX,BASE,SLOPE
      INTEGER  I,CMPT
      REAL     DEP,APPTOT,APPREM,FRACT,SLP2,FRCTOT

      FRCTOT=0.0
      SLP2= 0.
      CMPT= 0
      DEP = 0.0
      APPTOT=0.0
      APPREM=APPAMT
       
      do 
        CMPT= CMPT + 1
        FRACT=(SLOPE*(DEP+DELX(CMPT)/2.)+BASE)*DELX(CMPT)
        FRCTOT=FRCTOT+FRACT
        IF(FRCTOT.GT.1.00)FRACT=FRACT-(1.0-FRCTOT)
        SOILAP(CHEM,CMPT)=AMIN1(APPREM,APPAMT*FRACT)
        APPTOT=APPTOT+SOILAP(CHEM,CMPT)
        APPREM=APPAMT-APPTOT
        DEP=DEP+DELX(CMPT)
        if (DEP >= (DMAX-1.E-4)) exit
      end do
       
      do I=1,CMPT
          PESTR(CHEM,I)=PESTR(CHEM,I)+SOILAP(CHEM,I)/(DELX(I)*THETO(I))
          SPESTR(CHEM,I)= PESTR(CHEM,I)*THETO(I)/(THETO(I) +KD(CHEM,I)*BD(I) + (THETAS(I) - THETO(I))*KH(CHEM,I))
      end do
      
END SUBROUTINE pesticide_decreasing_distribution

!******************************************************************************************************************************
SUBROUTINE pesticide_uniform_distribution(APPAMT,DMAX,CHEM)
       use  constants_and_Variables, ONLY: NAPP,NC,NCMPTS,DELX,THETO,BD,thetas,KD,pestr,spestr,SOILAP,KH
       implicit none
      ! This routine distributes a chemical application uniformly to depth dmax.
      ! Used for CAM 1 ans 6 and Harvest
      
      ! APPAMT - total amount of chemical to distribute in the soil
      ! CHEM   - chemical id number (1-3)
      ! DMAX   - depth to which chemical should be applied;

      INTEGER  CHEM
      REAL     APPAMT,DMAX
      INTEGER  I,CMPT
      REAL     DEP,APPTOT,APPREM,FRACT,SLP2,FRCTOT

      FRCTOT=0.0
      SLP2= 0.
      CMPT= 0
      DEP = 0.0
      APPTOT=0.0
      APPREM=APPAMT
      
      do       
        CMPT= CMPT + 1
        FRACT=DELX(CMPT)/DMAX
        FRCTOT=FRCTOT+FRACT
        IF(FRCTOT.GT.1.00)FRACT=FRACT-(1.0-FRCTOT)
        SOILAP(CHEM,CMPT)=AMIN1(APPREM,APPAMT*FRACT)
        APPTOT=APPTOT+SOILAP(CHEM,CMPT)
        APPREM=APPAMT-APPTOT
        DEP=DEP+DELX(CMPT)
        if (DEP >= DMAX) exit
      end do  
       
      do I=1,CMPT
          PESTR(CHEM,I)=PESTR(CHEM,I)+SOILAP(CHEM,I)/(DELX(I)*THETO(I))
          SPESTR(CHEM,I)= PESTR(CHEM,I)*THETO(I)/(THETO(I) +KD(CHEM,I)*BD(I) + (THETAS(I) - THETO(I))*KH(CHEM,I))
      end do

 END SUBROUTINE pesticide_uniform_distribution
 
 !******************************************************************************************************************************
 SUBROUTINE pesticide_increasing_distribution(APPAMT,DMAX,BASE,SLOPE,CHEM)
      use  constants_and_Variables, ONLY: NAPP,NC,NCMPTS,DELX,THETO,BD,thetas,KD,pestr,spestr,SOILAP,KH
      implicit none
      !This routine distributes a chemical application down to an input depth increasing proportionally with depth.
      
      !APPAMT - total amount of chemical to distribute in the soil
      !CHEM   - chemical id number (1-3)
      !DMAX   - depth to which chemical should be applied;
      !BASE   - initial starting amount of application
      !SLOPE  - slope of linearly decreasing application

      INTEGER  CHEM
      REAL     APPAMT,DMAX,BASE,SLOPE
      INTEGER  I,CMPT
      REAL     DEP,APPTOT,APPREM,FRACT,SLP2,FRCTOT

      FRCTOT=0.0
      SLP2= 0.
      CMPT= 0
      DEP = 0.0
      APPTOT=0.0
      APPREM=APPAMT
      
      do     
        CMPT= CMPT + 1
        DEP=DEP+DELX(CMPT)        
        FRACT=(SLOPE*((DMAX-DEP)+DELX(CMPT)/2.)+BASE)*DELX(CMPT)
        FRCTOT=FRCTOT+FRACT
        IF(FRCTOT.GT.1.00)FRACT=FRACT-(1.0-FRCTOT)
        SOILAP(CHEM,CMPT)=AMIN1(APPREM,APPAMT*FRACT)
        APPTOT=APPTOT+SOILAP(CHEM,CMPT)
        APPREM=APPAMT-APPTOT
        IF(DEP>=(DMAX-1.E-4)) exit
      end do
             
      do I=1,CMPT
          PESTR(CHEM,I)=PESTR(CHEM,I)+SOILAP(CHEM,I)/(DELX(I)*THETO(I))
          SPESTR(CHEM,I)= PESTR(CHEM,I)*THETO(I)/(THETO(I) +KD(CHEM,I)*BD(I) + (THETAS(I) - THETO(I))*KH(CHEM,I))
      end do

END SUBROUTINE pesticide_increasing_distribution
  

SUBROUTINE pesticide_Tband_distribution(APPAMT,DMAX,CHEM,top2cm)
      use  constants_and_Variables, ONLY: NAPP,NC,NCMPTS,DELX,THETO,BD,thetas,KD,pestr,spestr,SOILAP,KH
      implicit none
      !This routine distributes ina T Band: Part in the top 2 cm and the remainder uniformly distributed, drft in this
      !case refers to fraction of application in the top 2 cm.
      
      !APPAMT - total amount of chemical to distribute in the soil
      !CHEM   - chemical id number (1-3)
      !DMAX   - depth to which chemical should be applied;

      INTEGER  CHEM
      REAL     APPAMT,DMAX
      INTEGER  I,CMPT
      REAL     DEP,APPTOT,APPREM,SLP2,FRCTOT
      real,intent(in) :: top2cm
      
      
      FRCTOT=0.0
      SLP2= 0.
      CMPT= 0
      DEP = 0.0
      APPTOT=0.0
      APPREM=APPAMT
      
       do
         CMPT= CMPT + 1
         DEP=DEP+DELX(CMPT)
         SLP2=DELX(CMPT)/2.
         IF(SLP2 > 1.)SLP2=1.0
         SOILAP(CHEM,CMPT)=AMIN1(APPREM,(SLP2*APPAMT*top2cm))
         APPTOT=APPTOT+SOILAP(CHEM,CMPT)
         APPREM=(APPAMT*top2cm)-APPTOT
         if (dep >= 1.95) exit
       end do 
            
       do  
          IF((DEP.GT.1.95).AND.(DEP.LT.2.05))APPREM=APPAMT-APPTOT
          CMPT= CMPT + 1
          DEP=DEP+DELX(CMPT)
          SLP2=DELX(CMPT)/(DMAX-2.)
          IF(SLP2.GT.1.)SLP2=1.0
        !  SOILAP(CHEM,CMPT)=AMIN1(APPREM,(SLP2*APPAMT*(1.0-DRFT(CHEM,NAPPC))))
          SOILAP(CHEM,CMPT)=AMIN1(APPREM,(SLP2*APPAMT*(1.0-top2cm)))
          APPTOT=APPTOT+SOILAP(CHEM,CMPT)
          APPREM=APPAMT-APPTOT
          if (DEP >= (DMAX-1.E-4)) exit
       end do
             
      do I=1,CMPT
          PESTR(CHEM,I)=PESTR(CHEM,I)+SOILAP(CHEM,I)/(DELX(I)*THETO(I))
          SPESTR(CHEM,I)= PESTR(CHEM,I)*THETO(I)/(THETO(I) +KD(CHEM,I)*BD(I) + (THETAS(I) - THETO(I))*KH(CHEM,I))
      end do

END SUBROUTINE pesticide_Tband_distribution
  
  
SUBROUTINE pesticide_atdepth_distribution(APPAMT,DMAX,CHEM)
      use constants_and_variables, ONLY: NAPP,NC,NCMPTS,DELX,THETO,BD,thetas,KD,pestr,spestr,SOILAP,KH
      implicit none
      !Places all pesticide at a specfic depth
      !APPAMT - total amount of chemical to distribute in the soil
      !CHEM   - chemical id number (1-3)
      
      INTEGER,intent(in) ::  CHEM
      REAL,intent(in)    ::  APPAMT,DMAX
      INTEGER  I,CMPT
      REAL     DEP
      
      CMPT= 0
      DEP = 0.0
      do
        CMPT= CMPT + 1
        DEP=DEP+DELX(CMPT)
        IF(DEP >= (DMAX-1.E-4)) exit
      end do
      SOILAP(CHEM,CMPT)=APPAMT
      
      do I=1,CMPT
          PESTR(CHEM,I)=PESTR(CHEM,I)+SOILAP(CHEM,I)/(DELX(I)*THETO(I))
          SPESTR(CHEM,I)= PESTR(CHEM,I)*THETO(I)/(THETO(I) +KD(CHEM,I)*BD(I) + (THETAS(I) - THETO(I))*KH(CHEM,I))
      end do
END SUBROUTINE pesticide_atdepth_distribution
     
  
  
end module Pesticide_Applications
 
 
 
    
    
    