 module readinputs
  implicit none
    contains
    

  !*****************************************************************************
   subroutine GETMET(status)
      !Read Met File and gets the Ordinal day (number of day of the year from Jan 1)          
      use constants_and_Variables, ONLY: NCMPTS,MetFileUnit,EchoFileUnit,TEMP,PEVP,PRECIP,SOLRAD,WIND,   &
                                          current_day,current_month,current_year,julday1900
      use utilities
      implicit none
      integer,intent(out) :: status  !status = -1 is end of file

      READ(MetFileUnit,'(1X,3I2,5F10.0)',IOSTAT=status) current_month,current_day,current_year, PRECIP,PEVP,TEMP,WIND,SOLRAD
        !THese are loaded into module:  current_month,current_day,current_year
             
        
        
      julday1900 = jd(current_year,current_month,current_day)  !Set the julian day into module
      
      if (status >=1) write( EchoFileUnit,*)"Metfile read failure"
      
   end subroutine   GETMET
    
!******************************************************************************************************************************
  SUBROUTINE PRZMRD_PRZM5
  !Reads and checks input data. 
      
      use  constants_and_Variables, ONLY:PRZMinputUnit, NAPP,NC,MXCPD,NCMPTS,EchoFileUnit,pfac, sfac,spt,vhtcap,thcond,itflag,  &
      idflag,albedo,emmiss,bbt,IRTYPE,PCDEPL,RATEAP,FLEACH, ANETD,cintcp,covmax,amxdr,ncpds, wfmax,htmax,                       &
      iyrem,iyrmat,iyrhar,Num_Crops,INCROP, icncn,ipscnd,INICRP,                                                 &
      AFIELD,IREG,USLEK,USLELS,USLEP,HL,SLP, USLEC,MNGN,BD,THETO,DELX,SAND,CLAY,THEFC,THEWP,CN_2,                               &
      NUSLEC,GDUSLEC,GMUSLEC,GYUSLEC,erflag,ncom2, IRFLAG, Num_delx,                          &
      DWRAT1,DSRAT1,DGRAT1,DWRATE,DSRATE,DGRATE,DEPI, KD,pestr,spestr,TAPP,appeff,Tband_top,FEXTRC,FILTRA,PLDKRT,DISP,         &
      PTRN12,PTRN13,PTRN23,DKRW12,DKRW13,DKRW23,ENPY,UPTKF,NAPS,DAIR,HENRYK,PLVKRT,OC,CAM,     &
      QFAC,TBASE,NCHEM,FAM,NHORIZ,SOL,THKNS,CONST,KDFLAG,THFLAG,INDX,NPLOTS,PCMC,             &
      IARG,IARG2,PLNAME,mode,PSTNAM,  &
      emergence_date, maturity_date, harvest_date, application_date,dispCoeff,  &
      runoff_effic,runoff_decline,runoff_extr_depth,erosion_decline,erosion_depth ,Num_hydro_factors,use_usleyears,  &
      metfileunit,maxFileLength,ISDAY,ISMON,ISTYR,TimeSeriesUnit,RunFilePath,  &
      FLAG1, FLAG2,FLAG3,FLAG4,calibrationflag,CalibrationDataUnit, CalibrationFilename, &
      UserSpecifiesDepth, user_irrig_depth, k_f_input, N_f_input,lowest_conc
                                            

      use utilities
      Use m_Wind
      implicit none

      INTEGER      KM    , dummy

      INTEGER      I,J,K,APM,APD,HAM,HAD,MAM, apy
      INTEGER      MAD,EMM,EMD
      
      INTEGER      CROPNO
      CHARACTER(len = 150):: note  !used for sending error messages

      LOGICAL      FATAL
      Logical      value_in_range
      integer ::  daymon(NC,Num_hydro_factors)
      Character(Len = maxFileLength)    :: Filename
      
      ! if CAM == 1,2,3 then force soil incorporation depth == 4 cm
      Real, Parameter :: cam123_soil_depth = 4.0 ! cm
      integer :: status
      
      I   = 80
      cam = 0
      sol = 0.0
      DKRW12= 0.0
      DKRW23= 0.0
      DKRW13= 0.0

      SPT     = 0.0
      SAND    = 0.0
      CLAY    = 0.0
      VHTCAP  = 0.0
      THCOND  = 0.0
      
      
      
      OPEN(Unit = PRZMinputUnit,FILE=(trim(adjustl(RunFilePath)) //'PRZM5.inp'),STATUS='OLD', IOSTAT=status)
      IF (status .NE. 0) THEN
          WRITE(EchoFileUnit,*)'Problem with PRZM INP file.'
      ENDIF 
      
      
      write(EchoFileUnit,*) 'Record A1' 
      CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,'(A)',IOSTAT = status) filename !Weather File

      if (status/=0) stop
      OPEN(Unit = metfileunit,FILE=trim(adjustl(filename)),STATUS='OLD', IOSTAT=status)
      IF (status .NE. 0) THEN
         note = 'Problem with met file.'
         CALL ERRCHK(note,FATAL)
      ENDIF                     
      note = 'Met File:'  // trim(adjustl(filename))
      WRITE(EchoFileUnit,*)  trim(note)
       
      read(metfileunit, '(1x,3I2)') ISDAY,ISMON,ISTYR
      rewind(metfileunit)
      WRITE(EchoFileUnit,*)  'Start Date = ',ISDAY,ISMON,ISTYR
      

       write(EchoFileUnit,*) 'Record A2:  PRZM Output Time Series File'
       CALL check_for_comment(PRZMinputUnit)
       READ(PRZMinputUnit,'(A)',IOSTAT = status) filename 
       
       if (status/=0) stop
       OPEN(Unit=TimeSeriesUnit,FILE=trim(adjustl(filename)), STATUS='UNKNOWN', IOSTAT=status)
       note= 'Time Series:' //  trim(Filename)
       WRITE(EchoFileUnit,*) trim(note) 

 
        write(EchoFileUnit,*) 'Record A3:  PRZM Options'
       CALL check_for_comment(PRZMinputUnit)
       READ(PRZMinputUnit,*,IOSTAT = status) FLAG1, FLAG2,FLAG3,FLAG4,calibrationFlag

       if (calibrationflag) then
         CALL check_for_comment(PRZMinputUnit)
         READ(PRZMinputUnit,'(A)',IOSTAT = status)  CalibrationFilename
         OPEN(Unit = CalibrationDataUnit,FILE=trim(adjustl( CalibrationFilename)),STATUS='OLD', IOSTAT=status)
         IF (status .NE. 0) THEN
             note = 'Problem with calibration data file.'
             CALL ERRCHK(note,FATAL)
         ENDIF  
         note = 'Calibration file:'  // trim(adjustl(filename))
         WRITE(EchoFileUnit,*)  trim(note)
       end if
       
      write(EchoFileUnit,*) 'Record 1'
      CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) PFAC,SFAC,ANETD,INICRP 
      if (status/=0) stop
                
      write(EchoFileUnit,*) 'Record 2'
       CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  ERFLAG
      if (status/=0) stop 

!    ERFLAG = 0 indicates that erosion will not be calculated
      if (ERFLAG.eq.1 .or. ERFLAG.gt.4 .or. ERFLAG.lt.0) then
         note = 'Erosion flag (ERFLAG) out of range'
         FATAL = .True.
         Call ERRCHK (note, FATAL)
      ENDIF 
      
      write(EchoFileUnit,*) 'Record 3'
      CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) USLEK,USLELS,USLEP,AFIELD,IREG,SLP,HL
      if (status/=0) stop            
   
!    crop information for individual crops
      write(EchoFileUnit,*) 'Record 4'
      CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  Num_Crops  !number of crops
      if (status/=0) stop

      IF (Num_Crops .GT. NC) THEN  ! check NDC, cannot be > 5
        note = "Too many crops"
        FATAL  = .TRUE.
        CALL ERRCHK(note, FATAL)
      ENDIF

!     reads crop parameters up to number of NDC(number of crops)
      write(EchoFileUnit,*) 'Record 5'
      do I=1,Num_Crops
        CALL check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status) ICNCN(I),CINTCP(I),AMXDR(I),COVMAX(I),WFMAX(I),HTMAX(I)
        if (status/=0) stop
      end do

      !--------- Erosion/Curve number Inputs ---------------------
      do KM=1,Num_Crops
                
          write(EchoFileUnit,*) 'Record 6'
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status)CROPNO,NUSLEC(KM) , use_usleyears
          if (status/=0) stop 
              
          write(EchoFileUnit,*) 'Record 7'
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status)(daymon(KM,I),I=1,NUSLEC(KM))
          if (status/=0) stop
          
          do  i = 1 ,NUSLEC(KM)
              GDUSLEC(KM,I)=daymon(km,i)/100
              GMUSLEC(KM,I)=daymon(km,i)-GDUSLEC(KM,I)*100
          end do 
                   
          if (use_usleyears==1) then
              write(EchoFileUnit,*) 'Record 7a'
              CALL check_for_comment(PRZMinputUnit)
              READ(PRZMinputUnit,*,IOSTAT = status)  (GYUSLEC(KM,I),I=1,NUSLEC(KM))
              if (status/=0) stop
          end if
          
          write(EchoFileUnit,*) 'Record 8'
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status)(USLEC(KM,I),I=1,NUSLEC(KM))
          if (status/=0) stop
              
          write(EchoFileUnit,*) 'Record 9'
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status) (MNGN(KM,I),I=1,NUSLEC(KM))   
          if (status/=0) stop
          
          write(EchoFileUnit,*) 'Record 10'
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status)(CN_2(KM,I),I=1,NUSLEC(KM))   
          if (status/=0) stop
      end do

      
      
      !-------------End Erosion Inputs ------------------------

      write(EchoFileUnit,*) 'Record 11'
      CALL check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  NCPDS
      if (status/=0) stop
      
      
      IF (NCPDS .GT. MXCPD) THEN
        WRITE(note,*) 'Number of Crop Periods is greater than ', MXCPD
        FATAL  = .TRUE.
        CALL ERRCHK(note, FATAL)
      ENDIF
     
      write(EchoFileUnit,*) 'Record 12'
      DO  I=1,NCPDS
          CALL check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status) EMD,EMM,IYREM(I),MAD,MAM,IYRMAT(I),HAD,HAM,IYRHAR(I),INCROP(I)
          if (status/=0) stop
          
          emergence_date(i) = jd(IYREM(I) ,emm,emd)
          maturity_date(i)  = jd(IYRMAT(I),mam,mad)
          harvest_date(i)   = jd(IYRHAR(I),ham,had)       
      end do
 
      write(EchoFileUnit,*) 'Record 13 Flags'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  IRFLAG,ITFLAG,IDFLAG  
      if (status/=0) stop
      
      
!     reads in irrigation parameters if IRFLAG > 0
      IF(IRFLAG .NE. 0)THEN
        write(EchoFileUnit,*) 'Record 14 Irrigation'
        call check_for_comment(PRZMinputUnit)
        !New Variables: user_irrig_depth, AutoSpecifyDepth
        READ(PRZMinputUnit,*,IOSTAT = status) IRTYPE,FLEACH,PCDEPL,RATEAP, UserSpecifiesDepth, user_irrig_depth
        if (status/=0) stop
        
        
        IF (IRTYPE ==2) then
            write (echofileunit,*) "Irrigation Type 2 is not supported in PRZM 5"
            stop
        end if    
        
         value_in_range = (0.0 <= PCDEPL) .And. (PCDEPL <= 1.0)
         If (.Not. value_in_range) Then
            note = "PCDEPL was not in the range 0 to 1.0; PCDEPL was reset to 0.5"
            FATAL = .False.
            CALL ERRCHK(note,FATAL)
            PCDEPL = 0.5
         End If
      END IF

      IF (ITFLAG .EQ. 1) THEN  !soil temperature simulation, if ITFLAG = 1
        write(EchoFileUnit,*) 'Record 15'
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status)(ALBEDO(I),I=1,12),EMMISS,uWind_Reference_Height
        if (status/=0) stop
        
!       reads in surface temperatures if ITFLAG = 1
        write(EchoFileUnit,*) 'Record 16'
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status)(BBT(I),I=1,12)
        if (status/=0) stop
        
!       reads in Q10FAC and TBASE
        write(EchoFileUnit,*) 'Record 17'
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status) QFAC,TBASE
        if (status/=0) stop
      ENDIF

      write(EchoFileUnit,*) 'Record 18'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) NHORIZ
      if (status/=0) stop
        
      IF (ITFLAG .EQ. 1) THEN
        do I=1,NHORIZ
          write(EchoFileUnit,*) 'Record 19  Soil Parameters with temperature'
          call check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT=status) dummy,THKNS(I),Num_delx(I),dispCoeff(i),BD(I),THETO(I),THEFC(I),THEWP(I),OC(I),  &
                                              SAND(I),CLAY(I),THCOND(I),SPT(I)
          if (status/=0) stop
        end do
      else      
         do I=1,NHORIZ
             write(EchoFileUnit,*) 'Record 19  Soil Parameters'
             call check_for_comment(PRZMinputUnit)
             READ(PRZMinputUnit,*,IOSTAT = status)  dummy,THKNS(I), num_delx(I), dispCoeff(i),BD(I),THETO(I),THEFC(I),THEWP(I),OC(I)
             if (status/=0) stop
         end do
      end if
   
  
!
!!     calculate NCOM2, number of compartments
!
!      IBGN = 0
!      JNN = 0
!      OLDJNN = 0
!      DO icount = 1, NHORIZ
!          ! JNN is number of comp in horizon
!          ! NCOMBE is beginning comp number for horiz icount, NCOMEN is end comp #
!          NCOMBE(icount) = OLDJNN + 1
!          JNN = INT(THKNS(icount)/DPN(icount)+0.5)
!          NCOMEN(icount) = JNN + OLDJNN
!          OLDJNN = JNN + OLDJNN
!          
!          do JLIN = 1, JNN
!            IBGN = IBGN + 1
!            DELX(IBGN) = DPN(icount)
!          end do
!      end do
!
!      NCOM2 = IBGN   !total number of counted compartments
!
!      IF (NCOM2+1 .GT. NCMPTS) THEN
!        WRITE(note,*) 'Too many compartments; Max =', NCMPTS
!        write(EchoFileUnit,*) note
!        stop
!      ENDIF

!     initial pesticide levels if desired
!     zero PESTR array here
          PESTR  = 0.0
          SPESTR = 0.0
      
      write(EchoFileUnit,*) 'Record 20  PRZM5 runoff extraction parameters'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) runoff_extr_depth,runoff_decline,runoff_effic     
      
      write(EchoFileUnit,*) 'Record 21  PRZM5 erosion extraction parameters'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  erosion_depth, erosion_decline
           
      !*********CHEMICAL INPUTS*************************************
      write(EchoFileUnit,*) 'Record C1'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) NAPS,NCHEM
      if (status/=0) stop
      
      IF (NCHEM .EQ. 0) NCHEM = 1

      IF (NAPS .GT. NAPP) THEN
        WRITE(note,*) 'Number of Pesticide applications is greater than: ', NAPP 
        FATAL  = .TRUE.
        CALL ERRCHK(note, FATAL)
      ENDIF
      
      write(EchoFileUnit,*) 'Record C2'
      DO I=1,NAPS
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status)APD,APM,APY,(CAM(K,I),DEPI(K,I),TAPP(K,I),APPEFF(K,I),Tband_top(K,I),K=1,NCHEM)
 
        if (status/=0) stop  
        application_date(i) = jd(APY, apm,apd)  !determine julian application date
      end do
      
      FAM=0
      if(any(cam==2).or.any(cam==3).or.any(cam==9).or.any(cam==10))then
         FAM=2
      end if
      
      write(EchoFileUnit,*) 'Record C3'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status)  FILTRA,(IPSCND(K),UPTKF(K),K=1,NCHEM)
      if (status/=0) stop
      
      IF(FAM.EQ.2)THEN  !foliar applications
        write(EchoFileUnit,*) 'Record C4'
        DO  K=1,NCHEM
         call check_for_comment(PRZMinputUnit)
         READ(PRZMinputUnit,*,IOSTAT = status) PLVKRT(K),PLDKRT(K),FEXTRC(K)
         if (status/=0) stop         
        end do
      ENDIF

      IF((FAM.EQ.2).AND.(NCHEM.GT.1))THEN
        write(EchoFileUnit,*) 'Record C5'
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status)  PTRN12,PTRN13,PTRN23
        if (status/=0) stop        
      ENDIF
      
      
    
      write(EchoFileUnit,*) 'Record C6'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) (DAIR(I),I=1,NCHEM),(HENRYK(I),I=1,NCHEM),(ENPY(I),I=1,NCHEM)
      if (status/=0) stop  
      
            
     !do I=1,NHORIZ
     !      write(EchoFileUnit,*) 'Record C7  Kd'
     !      call check_for_comment(PRZMinputUnit)
     !      READ(PRZMinputUnit,*,IOSTAT = status)(KD(K,I),K=1,NCHEM)
     !      if (status/=0) stop
     !end do
      

      do I=1,NHORIZ
           write(EchoFileUnit,*) 'Record C7  Kf'
           call check_for_comment(PRZMinputUnit)
           READ(PRZMinputUnit,*,IOSTAT = status)(k_f_input(K,I),K=1,NCHEM)
           if (status/=0) stop
      end do
      
      do I=1,NHORIZ
           write(EchoFileUnit,*) 'Record C7A  Freundlich N'
           call check_for_comment(PRZMinputUnit)
           READ(PRZMinputUnit,*,IOSTAT = status)(N_f_input(K,I),K=1,NCHEM)
           if (status/=0) stop
      end do
      
      write(EchoFileUnit,*) 'Record C7B  Concnetration below which isotherm is linear(mg/L)'
      call check_for_comment(PRZMinputUnit)
      READ(PRZMinputUnit,*,IOSTAT = status) lowest_conc 
 
      
      
      
     do I=1,NHORIZ
          write(EchoFileUnit,*) 'Record C8 Degradation Rates'
          call check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status)(DWRAT1(K,I),K=1,NCHEM),(DSRAT1(K,I),K=1,NCHEM),(DGRAT1(K,I),K=1,NCHEM)
          if (status/=0) stop

          do J=1,NCHEM
              DWRAT1(J,I)=EXP(DWRAT1(J,I))-1.   !rates are immediately converted to daily fractional decline
              DSRAT1(J,I)=EXP(DSRAT1(J,I))-1.
              DGRAT1(J,I)=EXP(DGRAT1(J,I))-1.
              
              DWRATE(J,I)=DWRAT1(J,I)
              DSRATE(J,I)=DSRAT1(J,I)
              DGRATE(J,I)=DGRAT1(J,I)
          end do
     end do

     do I=1,NHORIZ
          !read the molar prouction ratio of daughter to parent
             write(EchoFileUnit,*) 'Record C9'
             call check_for_comment(PRZMinputUnit)
             READ(PRZMinputUnit,*,IOSTAT = status) DKRW12(I),DKRW13(I),DKRW23(I)   !,DKRS12(I),DKRS13(I),DKRS23(I)  npt used
            
             if (status/=0) stop
      end do

 
      DISP(1,:) = dispcoeff
      DISP(2,:) = dispcoeff
      DISP(3,:) = dispcoeff
      

      
!****************TIME SERIES OUTPUT****************

      write(EchoFileUnit,*) 'Record U1'
        call check_for_comment(PRZMinputUnit)
        READ(PRZMinputUnit,*,IOSTAT = status) NPLOTS


      IF (NPLOTS .GT. 12) THEN
        note = 'NPLOTS is greater than 12, only 12 are used'
        FATAL  = .FALSE.
        CALL ERRCHK(note,FATAL)
        NPLOTS = 12
      ENDIF
      
      IF (NPLOTS .GT. 0) THEN
   
        DO I=1,NPLOTS
          write(EchoFileUnit,*) 'Record U2' 
          call check_for_comment(PRZMinputUnit)
          READ(PRZMinputUnit,*,IOSTAT = status) PLNAME(I),INDX(I),MODE(I),IARG(I),IARG2(I),CONST(I)
       end do
      ENDIF
 
      
      write(EchoFileUnit,*) 'End of record reading'



      
  END subroutine PRZMRD_PRZM5
         
   
  

  

end module readinputs